@@ -6,6 +6,7 @@
 
 #include <KItinerary/CountryDb>
 
+#include <config-kitinerary.h>
 #include <knowledgedb/alphaid.h>
 #include <knowledgedb/trainstationdb.h>
 
@@ -18,11 +19,6 @@ using namespace KItinerary;
 using namespace KItinerary::KnowledgeDb;
 
 namespace KItinerary { namespace KnowledgeDb {
-char *toString(Tz tz)
-{
-    using QTest::toString;
-    return toString(tzId(tz));
-}
 char *toString(CountryId c)
 {
     using QTest::toString;
@@ -30,6 +26,12 @@ char *toString(CountryId c)
 }
 }}
 
+char *toString(const QTimeZone &tz)
+{
+    using QTest::toString;
+    return toString(tz.id());
+}
+
 class KnowledgeDbTest : public QObject
 {
     Q_OBJECT
@@ -106,26 +108,21 @@ private Q_SLOTS:
     {
         auto station = KnowledgeDb::stationForIbnr(IBNR{1234567});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForIbnr({});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForIbnr(IBNR{8011160});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Berlin"));
         QCOMPARE(station.country, CountryId{"DE"});
 
         station = KnowledgeDb::stationForIbnr(IBNR{8501687});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Zurich"));
         QCOMPARE(station.country, CountryId{"CH"});
 
         // Aachen West, very close to the NL border, should be in DE timezone
         station = KnowledgeDb::stationForIbnr(IBNR{8000404});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Berlin"));
         QCOMPARE(station.country, CountryId{"DE"});
     }
 
@@ -133,15 +130,12 @@ private Q_SLOTS:
     {
         auto station = KnowledgeDb::stationForUic(UICStation{1234567});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForUic({});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForUic(UICStation{1001332});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Helsinki"));
         QCOMPARE(station.country, CountryId{"FI"});
     }
 
@@ -149,34 +143,27 @@ private Q_SLOTS:
     {
         auto station = KnowledgeDb::stationForSncfStationId({});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForSncfStationId(SncfStationId{"XXXXX"});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForSncfStationId(SncfStationId{"FRAES"});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Paris"));
         QCOMPARE(station.country, CountryId{"FR"});
 
         station = KnowledgeDb::stationForSncfStationId(SncfStationId{QStringLiteral("FRXYT")});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Paris"));
         QCOMPARE(station.country, CountryId{"FR"});
 
         station = KnowledgeDb::stationForSncfStationId(SncfStationId{"CHGVA"});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Zurich"));
         QCOMPARE(station.country, CountryId{"CH"});
         station = KnowledgeDb::stationForSncfStationId(SncfStationId{"FRHWO"}); // alias for CHGVA...
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Zurich"));
         QCOMPARE(station.country, CountryId{"CH"});
 
         station = KnowledgeDb::stationForSncfStationId(SncfStationId{"NLAMA"}); // vs. SNCB ID of NLASC
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Amsterdam"));
         QCOMPARE(station.country, CountryId{"NL"});
     }
 
@@ -184,15 +171,12 @@ private Q_SLOTS:
     {
         auto station = KnowledgeDb::stationForBenerailId({});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForBenerailId(BenerailStationId{"XXXXX"});
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForBenerailId(BenerailStationId{"NLASC"});
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Amsterdam"));
         QCOMPARE(station.country, CountryId{"NL"});
     }
 
@@ -257,20 +241,9 @@ private Q_SLOTS:
     {
         using namespace KnowledgeDb;
 
-        QCOMPARE(toQTimeZone(timezoneForCountry(CountryId{"DE"})), QTimeZone("Europe/Berlin"));
-        QCOMPARE(toQTimeZone(timezoneForCountry(CountryId{"FR"})), QTimeZone("Europe/Paris"));
-        QCOMPARE(toQTimeZone(timezoneForCountry(CountryId{"BR"})), QTimeZone());
-    }
-
-    void testCountryForTimezone()
-    {
-        using namespace KnowledgeDb;
-
-        QCOMPARE(countryForTimezone(Tz::Europe_Busingen), CountryId{"DE"});
-        QCOMPARE(countryForTimezone(Tz::America_Los_Angeles), CountryId{"US"});
-        QCOMPARE(countryForTimezone(Tz::Asia_Kuching), CountryId{"MY"});
-        QCOMPARE(countryForTimezone(Tz::Asia_Bangkok), CountryId{});
-        QCOMPARE(countryForTimezone(Tz::Asia_Ho_Chi_Minh), CountryId{"VN"});
+        QCOMPARE(timezoneForLocation(NAN, NAN, u"DE"), QTimeZone("Europe/Berlin"));
+        QCOMPARE(timezoneForLocation(NAN, NAN, u"FR"), QTimeZone("Europe/Paris"));
+        QCOMPARE(timezoneForLocation(NAN, NAN, u"BR"), QTimeZone());
     }
 
     void testTimezoneForLocation()
@@ -278,56 +251,61 @@ private Q_SLOTS:
         using namespace KnowledgeDb;
 
         // basic checks in all quadrants
-        QCOMPARE(timezoneForCoordinate(52.4, 13.1), Tz::Europe_Berlin);
-        QCOMPARE(timezoneForCoordinate(-8.0, -35.0), Tz::America_Recife);
-        QCOMPARE(timezoneForCoordinate(-36.5, 175.0), Tz::Pacific_Auckland);
-        QCOMPARE(timezoneForCoordinate(44.0, -79.5), Tz::America_Toronto);
+        QCOMPARE(timezoneForLocation(52.4, 13.1, {}), QTimeZone("Europe/Berlin"));
+        QCOMPARE(timezoneForLocation(-8.0, -35.0, {}), QTimeZone("America/Recife"));
+        QCOMPARE(timezoneForLocation(-36.5, 175.0, {}), QTimeZone("Pacific/Auckland"));
+        QCOMPARE(timezoneForLocation(44.0, -79.5, {}), QTimeZone("America/Toronto"));
 
         // Special case: Northern Vietnam has a Thai timezone
-        QCOMPARE(timezoneForCoordinate(21.0, 106.0), Tz::Asia_Bangkok);
+        QCOMPARE(timezoneForLocation(21.0, 106.0, {}), QTimeZone("Asia/Bangkok"));
 
         // Maastricht (NL), very close to the BE border
-        QCOMPARE(timezoneForLocation(50.8505, 5.6881, CountryId{}), Tz::Europe_Amsterdam);
-        QCOMPARE(timezoneForLocation(50.8505, 5.6881, CountryId{"NL"}), Tz::Europe_Amsterdam);
+        QCOMPARE(timezoneForLocation(50.8505, 5.6881, QString()), QTimeZone("Europe/Amsterdam"));
+        QCOMPARE(timezoneForLocation(50.8505, 5.6881, u"NL"), QTimeZone("Europe/Amsterdam"));
 
         // Aachen, at the BE/DE/NL corner
-        QCOMPARE(timezoneForLocation(50.7717, 6.04235, CountryId{}), Tz::Europe_Berlin);
-        QCOMPARE(timezoneForLocation(50.7717, 6.04235, CountryId{"DE"}), Tz::Europe_Berlin);
-        //QCOMPARE(timezoneForLocation(50.7727, 6.01565, CountryId{}), Tz::Europe_Brussels);
-        QCOMPARE(timezoneForLocation(50.7727, 6.01565, CountryId{"BE"}), Tz::Europe_Brussels);
+        QCOMPARE(timezoneForLocation(50.7717, 6.04235, QString()), QTimeZone("Europe/Berlin"));
+        QCOMPARE(timezoneForLocation(50.7717, 6.04235, u"DE"), QTimeZone("Europe/Berlin"));
+        //QCOMPARE(timezoneForLocation(50.7727, 6.01565, QString()), QTimeZone("Europe/Brussels"));
+        QCOMPARE(timezoneForLocation(50.7727, 6.01565, u"BE"), QTimeZone("Europe/Brussels"));
 
         // Geneva (CH), very close to the FR border
-        QCOMPARE(timezoneForLocation(46.23213, 6.10636, CountryId{"CH"}), Tz::Europe_Zurich);
+        QCOMPARE(timezoneForLocation(46.23213, 6.10636, u"CH"), QTimeZone("Europe/Zurich"));
 
         // Busingen (DE), enclosed by CH, and in theory its own timezone (which we ignore)
-        QCOMPARE(timezoneForLocation(47.69947, 8.68833, CountryId{"DE"}), Tz::Europe_Berlin);
-        QCOMPARE(timezoneForLocation(47.67904, 8.68813, {}), Tz::Europe_Zurich);
+        QCOMPARE(timezoneForLocation(47.69947, 8.68833, u"DE"), QTimeZone("Europe/Berlin"));
+        QCOMPARE(timezoneForLocation(47.67904, 8.68813, {}), QTimeZone("Europe/Zurich"));
 
         // Baarle, the ultimate special case, NL/BE differs house by house
-        QCOMPARE(timezoneForLocation(51.44344, 4.93373, CountryId{"BE"}), Tz::Europe_Brussels);
-        QCOMPARE(timezoneForLocation(51.44344, 4.93373, CountryId{"NL"}), Tz::Europe_Amsterdam);
-        bool ambiguous = false;
-        auto tz = timezoneForCoordinate(51.44344, 4.93373, &ambiguous);
-        QVERIFY(ambiguous);
-        QVERIFY(tz == Tz::Europe_Amsterdam || tz == Tz::Europe_Brussels);
+        QCOMPARE(timezoneForLocation(51.44344, 4.93373, u"BE"), QTimeZone("Europe/Brussels"));
+        QCOMPARE(timezoneForLocation(51.44344, 4.93373, u"NL"), QTimeZone("Europe/Amsterdam"));
+        const auto tz = timezoneForLocation(51.44344, 4.93373, {});
+        QVERIFY(tz == QTimeZone("Europe/Amsterdam") || tz == QTimeZone("Europe/Brussels"));
 
         // Eliat Airport (IL), close to JO, and with a minor timezone variation due to different weekends
-        QCOMPARE(timezoneForLocation(29.72530, 35.00598, CountryId{"IL"}), Tz::Asia_Jerusalem);
-        QCOMPARE(timezoneForLocation(29.60908, 35.02038, CountryId{"JO"}), Tz::Asia_Amman);
+        QCOMPARE(timezoneForLocation(29.72530, 35.00598, u"IL"), QTimeZone("Asia/Jerusalem"));
+        QCOMPARE(timezoneForLocation(29.60908, 35.02038, u"JO"), QTimeZone("Asia/Amman"));
 
         // Tijuana (MX), close to US, tests equivalent tz search in the neighbouring country
-        QCOMPARE(timezoneForLocation(32.54274, -116.97505, CountryId{"MX"}), Tz::America_Tijuana);
-        QCOMPARE(timezoneForLocation(32.55783, -117.04773, CountryId{"US"}), Tz::America_Los_Angeles);
+        QCOMPARE(timezoneForLocation(32.54274, -116.97505, u"MX"), QTimeZone("America/Tijuana"));
+        QCOMPARE(timezoneForLocation(32.55783, -117.04773, u"US"), QTimeZone("America/Los_Angeles"));
 
         // Cordoba (AR), AR has several sub-zones that are all equivalent
-        QCOMPARE(timezoneForLocation(-31.4, -64.2, CountryId{"AR"}), Tz::America_Argentina_Cordoba);
+#if HAVE_KI18N_LOCALE_DATA
+        QCOMPARE(timezoneForLocation(-31.4, -64.2, u"AR"), QTimeZone("America/Argentina/Buenos_Aires"));
+#else
+        QCOMPARE(timezoneForLocation(-31.4, -64.2, u"AR"), QTimeZone("America/Argentina/Cordoba"));
+#endif
 
         // polar regions
-        QCOMPARE(timezoneForCoordinate(-90.0, 0.0), Tz::Undefined);
-        QCOMPARE(timezoneForCoordinate(90.0, 0.0), Tz::Undefined);
+        QCOMPARE(timezoneForLocation(-90.0, 0.0, {}), QTimeZone());
+        QCOMPARE(timezoneForLocation(90.0, 0.0, {}), QTimeZone());
 
         // Hong Kong seems problematic on FreeBSD
-        QCOMPARE(timezoneForCoordinate(22.31600, 113.93688), Tz::Asia_Hong_Kong);
+        QCOMPARE(timezoneForLocation(22.31600, 113.93688, {}), QTimeZone("Asia/Hong_Kong"));
+
+        // coordinates not provided
+        QCOMPARE(timezoneForLocation(NAN, NAN, u"LU"), QTimeZone("Europe/Luxembourg"));
     }
 
     void testCountryFromCoordinate()
@@ -335,24 +313,31 @@ private Q_SLOTS:
         using namespace KnowledgeDb;
 
         // basic tests
-        QCOMPARE(countryForCoordinate(52.4, 13.1), CountryId{"DE"});
-        QCOMPARE(countryForCoordinate(-8.0, -35.0), CountryId{"BR"});
-        QCOMPARE(countryForCoordinate(-36.5, 175.0), CountryId{"NZ"});
-        QCOMPARE(countryForCoordinate(44.0, -79.5), CountryId{"CA"});
+        QCOMPARE(countryForCoordinate(52.4, 13.1), QLatin1String{"DE"});
+        QCOMPARE(countryForCoordinate(-8.0, -35.0), QLatin1String{"BR"});
+        QCOMPARE(countryForCoordinate(-36.5, 175.0), QLatin1String{"NZ"});
+        QCOMPARE(countryForCoordinate(44.0, -79.5), QLatin1String{"CA"});
 
         // ambiguous locations
-        QCOMPARE(countryForCoordinate(51.44344, 4.93373), CountryId{});
+        QCOMPARE(countryForCoordinate(51.44344, 4.93373), QString());
 
+#if HAVE_KI18N_LOCALE_DATA
         // special case: northern Vietnam has a non-VN timezone (not the case anywhere else in the world up to 2020a)
-        QCOMPARE(countryForCoordinate(21.0, 106.0), CountryId{});
-        QCOMPARE(countryForCoordinate(10.5, 107.0), CountryId{"VN"});
-        QCOMPARE(countryForCoordinate(13.7, 100.4), CountryId{});
+        QCOMPARE(countryForCoordinate(21.0, 106.0), QLatin1String("VN"));
+        QCOMPARE(countryForCoordinate(10.5, 107.0), QLatin1String{"VN"});
+        QCOMPARE(countryForCoordinate(13.7, 100.4), QLatin1String("TH"));
+#else
+        // special case: northern Vietnam has a non-VN timezone (not the case anywhere else in the world up to 2020a)
+        QCOMPARE(countryForCoordinate(21.0, 106.0), QString());
+        QCOMPARE(countryForCoordinate(10.5, 107.0), QLatin1String{"VN"});
+        QCOMPARE(countryForCoordinate(13.7, 100.4), QString());
 
         // disputed areas
-        QCOMPARE(countryForCoordinate(45.0, 34.0), CountryId{});
+        QCOMPARE(countryForCoordinate(45.0, 34.0), QString());
+#endif
 
         // overseas territories with separate ISO 3166-1 codes
-        QCOMPARE(countryForCoordinate(4.8, -52.3), CountryId{"GF"}); // could also be "FR"
+        QCOMPARE(countryForCoordinate(4.8, -52.3), QLatin1String{"GF"}); // could also be "FR"
     }
 
     void testUICCountryCodeLookup()
@@ -363,6 +348,7 @@ private Q_SLOTS:
         QCOMPARE(KnowledgeDb::countryIdForUicCode(0), CountryId{});
     }
 
+#if !HAVE_KI18N_LOCALE_DATA
     void testIso3Lookup()
     {
         using namespace KnowledgeDb;
@@ -370,32 +356,28 @@ private Q_SLOTS:
         QCOMPARE(KnowledgeDb::countryIdFromIso3166_1alpha3(CountryId3{"ITA"}), CountryId{"IT"});
         QCOMPARE(KnowledgeDb::countryIdFromIso3166_1alpha3(CountryId3{"FOO"}), CountryId{});
     }
+#endif
 
     void testIndianRailwaysStationCodeLookup()
     {
         auto station = KnowledgeDb::stationForIndianRailwaysStationCode(QString());
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
 
         station = KnowledgeDb::stationForIndianRailwaysStationCode(QStringLiteral("NDLS"));
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Asia/Kolkata"));
         QCOMPARE(station.country, CountryId{"IN"});
 
         station = KnowledgeDb::stationForIndianRailwaysStationCode(QStringLiteral("ndls"));
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
     }
 
     void testFinishStationCodeLookup()
     {
         auto station = KnowledgeDb::stationForVRStationCode(VRStationCode(QStringLiteral("HKI")));
         QVERIFY(station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone("Europe/Helsinki"));
 
         station = KnowledgeDb::stationForVRStationCode(VRStationCode(QStringLiteral("BLÄ")));
         QVERIFY(!station.coordinate.isValid());
-        QCOMPARE(toQTimeZone(station.timezone()), QTimeZone());
     }
 };
 
